@@ -3,11 +3,11 @@ import os
 import random
 import shutil
 import time
-from unittest import case
 
 import requests
 from selenium import webdriver
 from loguru import logger
+from selenium.common import TimeoutException
 from selenium.webdriver import ActionChains
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support.wait import WebDriverWait
@@ -43,7 +43,6 @@ class SeleniumSession:
             self.headers.update(headers)
 
     def init_driver(self, driver_type):
-
         match driver_type:
             case "firefox":
                 option = webdriver.FirefoxOptions()
@@ -139,22 +138,63 @@ class SeleniumSession:
                 from selenium.webdriver.firefox.service import Service
                 self.driver = webdriver.Firefox(service=Service(executable_path=self.driver_path), options=option)
 
-    def get(self, url, **kwargs):
+    def get(self, url, headers=None, is_to_driver=True, is_refresh=True, **kwargs):
+        header_data = self.headers.copy()
+        if headers is not None:
+            header_data.update(headers)
+
         logger.debug(f"session请求(method: GET): {url}")
-        res = self.session.get(url, headers=self.headers, **kwargs)
-        self.cookies_to_driver()
+
+        res = self.session.get(url, headers=header_data, **kwargs)
+        if is_to_driver:
+            self.cookies_to_driver(is_refresh=is_refresh)
         return res
 
-    def post(self, url, data=None, json=None, **kwargs):
+    def get_img_base64_from_img_tag(self, value, by=By.CSS_SELECTOR, timeout=60):
+        js_code = """
+                   const canvas = document.createElement('canvas');
+                   canvas.width = arguments[0].width;
+                   canvas.height = arguments[0].height;
+                   const context = canvas.getContext('2d');
+                   context.drawImage(arguments[0], 0, 0);
+                   return canvas.toDataURL('image/png').split(',')[1];
+                   """
+        img = self.find_element(value, by=by, timeout=timeout)
+        return self.execute_script(js_code, img)
+
+    def get_action_chains(self):
+        return ActionChains(self.driver)
+
+    def execute_script(self, script, *args):
+        return self.driver.execute_script(script, *args)
+
+    def find_element(self, value, by=By.CSS_SELECTOR, timeout=60):
+        try:
+            element = WebDriverWait(self.driver, timeout).until(
+                EC.presence_of_element_located((by, value))
+            )
+            return element
+        except TimeoutException:
+            logger.error(f"未能找到元素{value}")
+
+    def post(self, url, data=None, json=None, headers=None, is_to_driver=True, is_refresh=True, **kwargs):
+        header_data = self.headers.copy()
+        if headers is not None:
+            header_data.update(headers)
         logger.debug(f"session请求(method: POST): {url}")
-        res = self.session.post(url, data=data, json=json, headers=self.headers, **kwargs)
-        self.cookies_to_driver()
+        res = self.session.post(url, data=data, json=json, headers=header_data, **kwargs)
+        if is_to_driver:
+            self.cookies_to_driver(is_refresh=is_refresh)
         return res
 
-    def request(self, url, method, **kwargs):
+    def request(self, url, method, headers=None, is_to_driver=True, is_refresh=True, **kwargs):
+        header_data = self.headers.copy()
+        if headers is not None:
+            header_data.update(headers)
         logger.debug(f"session请求(method: {method}): {url}")
-        res = self.session.request(method=method, url=url, headers=self.headers, **kwargs)
-        self.cookies_to_driver()
+        res = self.session.request(method=method, url=url, headers=header_data, **kwargs)
+        if is_to_driver:
+            self.cookies_to_driver(is_refresh=is_refresh)
         return res
 
     def get_session_cookies_to_dict(self):
@@ -167,10 +207,11 @@ class SeleniumSession:
             })
         return selenium_cookies
 
-    def cookies_to_driver(self):
+    def cookies_to_driver(self, is_refresh=True):
         for i in self.get_session_cookies_to_dict():
             self.driver.add_cookie(i)
-        self.driver.refresh()
+        if is_refresh:
+            self.driver.refresh()
 
     def selenium_get(self, url):
         logger.debug(f"浏览器请求: {url}")
@@ -281,7 +322,6 @@ class SeleniumSession:
             case _:
                 return dom.css(value)
 
-
     @staticmethod
     def on_input(des=""):
         return input(des)
@@ -328,6 +368,13 @@ class SeleniumSession:
         """
         self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
 
+    def quit(self):
+        """
+        退出浏览器
+        :return:
+        """
+        self.driver.quit()
+
     def save_cookies(self, save_path):
         logger.info(f"正在保存cookie中...")
         with open(save_path, "w") as f:
